﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;
using System.Runtime.Serialization.Formatters.Binary;

/*this is a sample project showing an easy way to save application config settings in an
  easy to use lightweight code format. You can extend yout config structure as you develop
  and not loose previous saved cfg settings. Works through object serialization and is trivial
  to incorporate a basic encryption on the config data to frustrate disclosure/tampering.

  this is much cleaner than MS bloated configuration manager crap and XML file puke. which
  i have had problems with.
 
 - Site: http://sandsprite.com  
 - Author: dzzie
   (note RC4 implementation was taken off the web)
 
 */

namespace ConfigTest
{
    class Program
    {
        private static void w(string msg) { Console.WriteLine(msg); }

        static void Main(string[] args)
        {

            CConfig cfg = new CConfig();
            w("Loaded config file: " + cfg.cfgFile);
            w("\ndefault string1: " + cfg.config.string1);

            cfg.config.string1= "Updated string 1";
            cfg.config.string3 = "Blahblah";
            cfg.SaveConfig();
            
            w("Config now updated. reloading");
            cfg.LoadConfig();

            w("Config loaded, default string1: " + cfg.config.string1);
            w("\nPress any key to exit...");
            Console.ReadKey();

        }
    }

    public class CConfig
    {
        [Serializable()]
        public struct MyConfig
        {
            public string string1;
            public string string2;
            public bool bool1;
            public byte[] bytes1;
            public string string3;
        }

        public MyConfig config;
        public string cfgFile;
        private byte[] cfgPass = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
        
        public CConfig()
        {

            string fname = @"\cfg.bin";
            
            //you could also have it always scan current dir, and test up sequentially till it found it..etc.
            if(System.Diagnostics.Debugger.IsAttached){ //running in IDE \bin\debug
                cfgFile = Directory.GetParent(Assembly.GetExecutingAssembly().Location).FullName;
                cfgFile = cfgFile.Replace(@"\bin\Debug", "");
            }else{
                cfgFile = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
                if (cfgFile.IndexOf(@"\bin\Release") > 0) cfgFile = cfgFile.Replace(@"\bin\Release", "");
            }

            cfgFile+= fname;
           
            if (!File.Exists(cfgFile))
            {
                Console.WriteLine("Could not locate cfg.bin? is this the first run? loading defaults...");

                config.bool1 = true;
                config.bytes1 = cfgPass;
                config.string1 = "This is string 1";
                config.string2 = "This is string 2!";
                config.string3 = "This is string 3";

                SaveConfig();
            }
            else
            {
                LoadConfig();
            }


        }

        public void SaveConfig()
        {

            BinaryFormatter formatter = new BinaryFormatter();
            MemoryStream stream = new MemoryStream();
            formatter.Serialize(stream, config);
            Byte[] buffer = stream.ToArray();

            if (File.Exists(cfgFile)) File.Delete(cfgFile);

            RC4(ref buffer, cfgPass);

            FileStream fs = File.Create(cfgFile, 2048, FileOptions.None);
            BinaryWriter bw = new BinaryWriter(fs);
            bw.Write(buffer);
            bw.Close();
            fs.Close();

        }

        public void LoadConfig()
        {

            byte[] buf = File.ReadAllBytes(cfgFile);
            RC4(ref buf, cfgPass);

            BinaryFormatter formatter2 = new BinaryFormatter();
            MemoryStream stream2 = new MemoryStream(buf);
            object o = formatter2.Deserialize(stream2);
            config = (MyConfig)o;

        }

        private void RC4(ref Byte[] bytes, Byte[] key)
        {
            Byte[] s = new Byte[256];
            Byte[] k = new Byte[256];
            Byte temp;
            int i, j;

            for (i = 0; i < 256; i++)
            {
                s[i] = (Byte)i;
                k[i] = key[i % key.GetLength(0)];
            }

            j = 0;
            for (i = 0; i < 256; i++)
            {
                j = (j + s[i] + k[i]) % 256;
                temp = s[i];
                s[i] = s[j];
                s[j] = temp;
            }

            i = j = 0;
            for (int x = 0; x < bytes.GetLength(0); x++)
            {
                i = (i + 1) % 256;
                j = (j + s[i]) % 256;
                temp = s[i];
                s[i] = s[j];
                s[j] = temp;
                int t = (s[i] + s[j]) % 256;
                bytes[x] ^= s[t];
            }
        }



    }
}
